@@ -1,10 +1,13 @@
 import abc
-import logging
 from past.builtins import long
+
+import logging
 import requests
 from requests import exceptions
 import six
-import xmltodict
+
+from .util import ApiXmlHelper
+from .exceptions import KempTechApiException
 
 
 requests.packages.urllib3.disable_warnings()
@@ -49,7 +52,7 @@ class HttpClient(object):
                                             params=parameters,
                                             verify=False)
             if 400 < response.status_code < 500:
-                raise KempTechApiError(code=response.status_code)
+                raise KempTechApiException(code=response.status_code)
             else:
                 response.raise_for_status()
         except exceptions.ConnectionError:
@@ -66,7 +69,7 @@ class HttpClient(object):
             raise
         except exceptions.HTTPError:
             log.exception("A HTTP error occurred with request to %s.", cmd_url)
-            raise KempTechApiError(msg=response.text)
+            raise KempTechApiException(msg=response.text)
         except exceptions.RequestException:
             log.exception("An error occurred with request to %s.", cmd_url)
             raise
@@ -184,14 +187,14 @@ class LoadMaster(HttpClient):
         except exceptions.RequestException:
             log.exception("An unknown error occurred with request to %s",
                           url)
-            raise KempTechApiError(response.text)
+            raise KempTechApiException(response.text)
 
     def stats(self):
         response = self._get('stats')
         if ApiXmlHelper.is_successful(response):
             return ApiXmlHelper.parse_to_dict(response)
         else:
-            raise KempTechApiError(ApiXmlHelper.get_error_msg(response))
+            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
 
     def update_firmware(self, file):
         response = self._post('installpatch', file)
@@ -205,73 +208,6 @@ class LoadMaster(HttpClient):
         response = self._get('reboot')
         return ApiXmlHelper.is_successful(response)
 
-
-class KempTechApiError(Exception):
-    """Raised if HTTP request has failed."""
-
-    def __init__(self, msg=None, code=None):
-        if msg is not None:
-            # 400 Errors should be handled here. This will pass the error
-            # message given by the LoadMaster API and show it as the exception
-            # message in the traceback.
-            message = ApiXmlHelper.get_error_msg(msg)
-        else:
-            # 400+ shouldn't be handled here, but cover it anyway. Should just
-            # re-raise whatever requests throws instead of depending on this.
-            # However, 400 itself should be handled, just not in this block.
-            # 404 is the only truly useful one here so far.
-            if code == 400:
-                message = "400 Mandatory parameter missing from request."
-            elif code == 401:
-                message = "401 Client Error: Authorization required."
-            elif code == 403:
-                message = "403 Incorrect permissions."
-            elif code == 404:
-                message = ("404 Not found. "
-                           "Ensure the API is enabled on the LoadMaster.")
-            elif code == 405:
-                message = "405 Unknown command."
-            else:
-                message = "An unknown error has occurred."
-        super(KempTechApiError, self).__init__(message)
-
-
-class ApiXmlHelper(object):
-    """Encapsulate the awful XML response and provide helpful functions."""
-    @classmethod
-    def get_success_msg(cls, xml):
-        return cls._get_xml_field(xml, "Success")
-
-    @classmethod
-    def get_error_msg(cls, xml):
-        return cls._get_xml_field(xml, "Error")
-
-    @classmethod
-    def is_successful(cls, xml):
-        """Return True if xml response contains a success, else false."""
-        if ApiXmlHelper.get_success_msg(xml):
-            return True
-        else:
-            return False
-
-    @classmethod
-    def get_data_field(cls, xml, field):
-        return cls._get_xml_field(xml, "Data", data_field=field)
-
-    @classmethod
-    def parse_to_dict(cls, xml):
-        """Return the XML as an OrderedDict."""
-        return xmltodict.parse(xml)
-
-    @classmethod
-    def _get_xml_field(cls, xml, field, data_field=None):
-        xml_dict = xmltodict.parse(xml)
-        try:
-            if data_field is None:
-                msg = xml_dict.get("Response").get(field)
-            else:
-                data = xml_dict.get("Response").get("Success").get(field)
-                msg = data.get(data_field)
-        except KeyError:
-            return False
-        return msg
+    def get_sdn_controller(self):
+        response = self._get('getsdncontroller')
+        return ApiXmlHelper.is_successful(response)
